(*
 * lexer.mll
 *
 * ocamlscheme
 *
 * Copyright (C) 2001-2004 Mark Probst
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *)

{
open Parser
exception Eof

let extract_string s = 
  assert( String.length s >= 2 );
  let shift = ref false
  and idx = ref ~-1 in
  let sub =
	String.sub s 1 (String.length s - 2) in
    let convert =
      function
	| '\\' when not !shift ->
	    shift := true
	| c ->
	    incr idx;
	    let nc =
	      if !shift then
		match c with
		  | 'n' -> '\n'
		  | 'b' -> '\b'
		  | 'r' -> '\r'
		  | _ -> c
	      else c in
	      shift := false;
	      String.unsafe_set sub !idx nc 
    in 
      String.iter convert sub; 
      String.sub sub 0 (succ !idx)

  let incr_linenum lexbuf =
    let pos = lexbuf.Lexing.lex_curr_p in
    lexbuf.Lexing.lex_curr_p <- { pos with
      Lexing.pos_lnum = pos.Lexing.pos_lnum + 1;
      Lexing.pos_bol = pos.Lexing.pos_cnum;
    }
  ;;
}

let token = [' ' '\t' '\r']+
let return = ['\n']
let intnum = ['-']*['0'-'9']+
let realnum = ['-']*['0'-'9']+['.']['0'-'9']+
let string_element = [^ '"' '\\'] | ("\\" _)
let string = '"' string_element * '"'
let symbol = [^'(' ')' '0' - '9' ' ' '\t' '\n' '.' '\''][^' ' '\t' '\n' '(' ')']*
let comment = [';'] [^'\n']* ['\n']

rule token = parse
    token          			  { token lexbuf }
  | return          			  { incr_linenum lexbuf;  token lexbuf }
  | intnum          		  { INT(int_of_string(Lexing.lexeme lexbuf)) }
  | realnum					  { FLOAT(float_of_string(Lexing.lexeme lexbuf)) }
  | string					  { STRING(extract_string(Lexing.lexeme lexbuf)) }
  | '('                       { LPAREN }
  | ')'                       { RPAREN }
  | '.'                       { DOT }
  | '\''                      { QUOTE }
  | symbol    			{ SYMBOL(Lexing.lexeme lexbuf) }
  | comment		      { token lexbuf }
  | eof                       { raise Eof }
